/* --*-C-*-- */
/* $Id: gimev_options.c.pl,v 1.6 2002/06/19 06:02:49 itz Exp $ */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <getopt.h>
#include <sys/types.h>
#include <stdio.h>
#include <assert.h>
#include <string.h>

#include "gimev_options.h"
#include "gim_public.h"

static struct option gimev_options[] = {
{ "device", required_argument, 0, 'd' },
{ "event_mask", required_argument, 0, 'e' },
{ "help", no_argument, 0, 'h' },
{ "max_mod", required_argument, 0, 'M' },
{ "min_mod", required_argument, 0, 'm' },
{ "pass_mask", required_argument, 0, 'p' },
{ "pointer", no_argument, 0, 'P' },
{ "terminal", required_argument, 0, 't' },

  { 0, 0, 0, 0 }
};

static const char gimev_docstrings[] = 
"A simple example gim client.\n"
"Usage: gimev [OPTION]... \n\n"
" -d, --device" "=DEVICE" "        " "Mouse attached to DEVICE (can be a symlink)"
 "\n" 
" -e, --event_mask" "=EVENT_MASK" " " "Ignore events not in EVENT_MASK"
 "\n" 
" -h, --help" "                 " "Print help information successfully"
 "\n" 
" -M, --max_mod" "=MAX_MOD" "      " "Ignore events with keyboard modifiers not in MAX_MOD"
 "\n" 
" -m, --min_mod" "=MIN_MOD" "      " "Ignore events without keyboard modifiers in MIN_MOD"
 "\n" 
" -p, --pass_mask" "=PASS_MASK" "  " "Pass events in PASS_MASK to root gim clients"
 "\n" 
" -P, --pointer" "              " "Draw mouse pointer during clicks and drags"
 "\n" 
" -t, --terminal" "=TERMINAL" "    " "Listen for mouse events on TERMINAL"
 "\n" 

  
"\nReport bugs to itz@speakeasy.org.\n"
;

void
print_gimev_version_and_copy()
{
  fprintf(stdout, "gimev %s\n", VERSION);
  fprintf(stdout, "Copyright (C) 2002 Ian Zimmerman.\ngim comes with NO WARRANTY, to the extent permitted by law.\nYou may redistribute copies of gim under the terms of the GNU\nGeneral Public License.  For more information about these matters,\nsee the file named GPL.\n");
  exit(0);
}

char* opt_device = 0;
char* opt_event_mask = 0;
int opt_help = 0;
char* opt_max_mod = 0;
char* opt_min_mod = 0;
char* opt_pass_mask = 0;
int opt_pointer = 0;
char* opt_terminal = 0;


void
parse_gimev_options(int argc, char** argv)
{
  int c;
  int i;
  static char optlist[] = "d:e:hM:m:p:Pt:";

  while ((c = getopt_long(argc, argv, optlist, gimev_options, &i)) != -1) {
    switch(c) {
      
	case 'd': opt_device = optarg; break;

	case 'e': opt_event_mask = optarg; break;

	case 'h': opt_help = 1; break;

	case 'M': opt_max_mod = optarg; break;

	case 'm': opt_min_mod = optarg; break;

	case 'p': opt_pass_mask = optarg; break;

	case 'P': opt_pointer = 1; break;

	case 't': opt_terminal = optarg; break;

	case '?':
	default:
	print_gimev_docstrings(1);
    } /*switch*/
  } /*while*/
}

int
print_gimev_docstrings (int exit_code)
{
  fprintf(stdout, gimev_docstrings);
  exit(exit_code);
  return 0;                     /* not reached */
}

int
process_gimev_command (int fd, char* buf, size_t bufsize)
{
  int status;
  struct gim_client_masks m;
  char* p;

  p = strtok(buf, " \t\n\r");
  if (p == 0) { strncpy(buf, "?", bufsize); return 0; }
  if (strcmp(p, "devinfo") == 0) return gim_client_devinfo(fd, buf, bufsize);
  if (strcmp(p, "disable_paste") == 0) { strncpy(buf, "ok", bufsize); return gim_client_disable_paste(fd); }
  if (strcmp(p, "handle") == 0) {
    status = gim_client_get_masks(fd, &m);
    if (status < 0) return status;
    p = strtok(0, " \t\n\r");
    m.handle = strtoul(p, 0, 0);
    strncpy(buf, "ok", bufsize);
    return gim_client_set_masks(fd, &m);
  }
  if (strcmp(p, "maxmod") == 0) {
    status = gim_client_get_masks(fd, &m);
    if (status < 0) return status;
    p = strtok(0, " \t\n\r");
    m.maxmod = strtoul(p, 0, 0);
    strncpy(buf, "ok", bufsize);
    return gim_client_set_masks(fd, &m);
  }
  if (strcmp(p, "minmod") == 0) {
    status = gim_client_get_masks(fd, &m);
    if (status < 0) return status;
    p = strtok(0, " \t\n\r");
    m.minmod = strtoul(p, 0, 0);
    strncpy(buf, "ok", bufsize);
    return gim_client_set_masks(fd, &m);
  }
  if (strcmp(p, "pass") == 0) {
    status = gim_client_get_masks(fd, &m);
    if (status < 0) return status;
    p = strtok(0, " \t\n\r");
    m.pass = strtoul(p, 0, 0);
    strncpy(buf, "ok", bufsize);
    return gim_client_set_masks(fd, &m);
  }
  if (strcmp(p, "start") == 0) { strncpy(buf, "ok", bufsize); return gim_client_start(fd); }
  if (strcmp(p, "stop") == 0) { strncpy(buf, "ok", bufsize); return gim_client_stop(fd); }
  if (strcmp(p, "servinfo") == 0) return gim_client_servinfo(fd, buf, bufsize);

  strncpy(buf, "?", bufsize); return 0;
}

void
print_gimev_help (void)
{
    fprintf(stdout, "%s: %s\n", "devinfo", "Print mouse device information (copied straight from libgii)");
  fprintf(stdout, "%s: %s\n", "disable_paste", "Tell server to disable pasting until a fresh selection is made");
  fprintf(stdout, "%s ARG: %s\n", "handle", "Change handled event mask to ARG");
  fprintf(stdout, "%s ARG: %s\n", "maxmod", "Change maximum modifier mask to ARG");
  fprintf(stdout, "%s ARG: %s\n", "minmod", "Change minimum modifier mask to ARG");
  fprintf(stdout, "%s ARG: %s\n", "pass", "Change passed event mask to ARG");
  fprintf(stdout, "%s: %s\n", "start", "Start receiving mouse events");
  fprintf(stdout, "%s: %s\n", "stop", "Stop receiving mouse events");
  fprintf(stdout, "%s: %s\n", "servinfo", "Print gim server information, including protocol version");

}
